const path = require("path");
const fs = require('fs');
const TerserJSPlugin = require('terser-webpack-plugin');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const OptimizeCSSAssetsPlugin = require('optimize-css-assets-webpack-plugin');

let getEntries = () => {
    const srcPath = "./src/scss";

    let entries = {};
    fs.readdirSync(srcPath).forEach(dir => {
        fs.readdirSync(path.join(srcPath, dir)).forEach(file => {
            if (!file.startsWith("_")) {
                entries[file.replace(".scss", "")] = "./" + path.join(srcPath, dir, file);
            }
        })
    });
    return entries;
};

var config = {
    entry: getEntries(),
    optimization: {
        minimizer: [new TerserJSPlugin({}), new OptimizeCSSAssetsPlugin({})],
    },
    plugins: [
        new MiniCssExtractPlugin({
            // Options similar to the same options in webpackOptions.output
            // both options are optional
            filename: '[name].css',
            chunkFilename: '[id].css',
        }),
    ],
    module: {
        rules: [
            {
                test: /\.(sa|sc|c)ss$/,
                use: [
                    {
                        loader: MiniCssExtractPlugin.loader,
                        options: {
                            hmr: false,
                            // if hmr does not work, this is a forceful method.
                            reloadAll: true,
                        },
                    },
                    'css-loader',
                    'postcss-loader',
                    'sass-loader',
                ],
            },
        ],
    },
};

module.exports = (env, argv) => {

    if (argv.mode === 'development') {
        config.watch = true;
        config.module.rules[0].use[0].options.hmr = true;
    }

    return config;
};
